@@ -54,11 +54,6 @@ class SimilarNamesInJob(Error):
     pass
 
 
-class ConnectionIdNotFound(Error):
-    """Using Cloud Object Storage requires a connection, but none was found"""
-    pass
-
-
 class NoCredentialsToCreateSpace(Error):
     """Need to create a space, but credentials are incomplete to allow that"""
     pass
@@ -145,9 +140,7 @@ class _CredentialsProvider:
     def check_credentials(self, wml_cred_str, url, region):
         assert type(wml_cred_str) is str
         if not wml_cred_str:
-            self._logger.error('WML credentials must not be an empty string.')
-            self.usage()
-            raise InvalidCredentials
+            raise InvalidCredentials('WML credentials must not be an empty string.')
         wml_credentials = ast.literal_eval(wml_cred_str)
         assert type(wml_credentials) is dict
         assert (self.APIKEY in wml_credentials or self.TOKEN in wml_credentials)
@@ -157,23 +150,19 @@ class _CredentialsProvider:
             assert type(wml_credentials[self.TOKEN]) is str
         if region:
             if url:
-                self._logger.error(f"You must not specify both '{self.URL}' and '{self.REGION}'.")
-                raise InvalidCredentials
+                raise InvalidCredentials(f"You must not specify both '{self.URL}' and '{self.REGION}'.")
             wml_credentials[self.REGION] = region
             # Setting a region must clear the URL, otherwise there will be an
             # ambiguity (and therefore an error) just below
-            wml_credentials.pop(self.URL)
+            wml_credentials.pop(self.URL, None)
         if self.REGION in wml_credentials:
             region = wml_credentials[self.REGION]
             if self.URL in wml_credentials:
-                self._logger.error(f"WML credentials must not have both '{self.URL}' and '{self.REGION}'.")
-                self.usage()
-                raise InvalidCredentials
+                raise InvalidCredentials(f"WML credentials must not have both '{self.URL}' and '{self.REGION}'.")
             try:
                 wml_credentials[self.URL] = self.REGION_TO_URL[region]
             except KeyError:
-                self._logger.error(f"Unknown region '{region}'.")
-                raise InvalidCredentials
+                raise InvalidCredentials(f"Unknown region '{region}'.")
             wml_credentials.pop(self.REGION)
         if url:
             # The url specified takes priority over the one in the credentials, if any.
@@ -182,8 +171,7 @@ class _CredentialsProvider:
         assert type(wml_credentials[self.URL]) is str
         url = wml_credentials[self.URL]
         if not url:
-            self._logger.error('URL must not be an empty string.')
-            raise InvalidCredentials
+            raise InvalidCredentials('URL must not be an empty string.')
         if url[-1] == '/':
             self._logger.warning('URL should not have a \'/\' at the end.')
             wml_credentials[self.URL] = url[:-1]
@@ -203,9 +191,7 @@ class _CredentialsProvider:
             self._logger.debug(f'Looking for credentials file name in environment variable {var_file_name}...')
             wml_cred_str = self._read_wml_credentials_from_file(os.environ[var_file_name])
         else:
-            print(f'Environment variables ${var_name} or ${var_file_name} not found.')
-            self.usage()
-            raise InvalidCredentials
+            raise InvalidCredentials(f'Environment variables ${var_name} or ${var_file_name} not found.')
 
         return wml_cred_str
 
@@ -624,6 +610,7 @@ class DOWMLLib:
             # Save the filter in a global variable for our new function to find it
             _the_filter = with_filter
             # Save the pointer to the original code
+            # noinspection PyProtectedMember
             _the_old_params = client._params
             # and replace it with our new function
             client._params = _new_params
@@ -694,6 +681,7 @@ class DOWMLLib:
         ws_url = client.PLATFORM_URLS_MAP.get(wml_url)
         if not ws_url:
             everything_ok_so_far = False
+        url = None  # Only required to silence the editor's warning
         if everything_ok_so_far:
             try:
                 platform_job_id = job_details['entity']['platform_job']['job_id']
@@ -702,6 +690,7 @@ class DOWMLLib:
             except KeyError:
                 everything_ok_so_far = False
         if everything_ok_so_far:
+            # noinspection PyProtectedMember
             r = requests.delete(url, headers={'Authorization': f'Bearer {client.service_instance._get_token()}',
                                               'Content-Type': 'application/json',
                                               'cache-control': 'no-cache'})
@@ -1028,7 +1017,7 @@ class DOWMLLib:
             },
             cdd_inputdata: [],
             cdd_outputdata: [
-                {'id': '.*\\.*'}
+                {'id': '.*'}
             ]
         }
         if self.outputs == 'assets':
@@ -1206,9 +1195,13 @@ class DOWMLLib:
                 }
             }
             # Create the space
-            space = client.spaces.store(meta_props=metadata)
-            self._logger.debug('Space created')
-            space_id = client.spaces.get_uid(space)
+            # We want the space to be ready as soon as the code returns from
+            # spaces.store(), so we use background_mode=False. In addition, this
+            # gives us error checking which doesn't happen in default mode.
+            space_details = client.spaces.store(meta_props=metadata, background_mode=False)
+            state = space_details['entity']['status'].get('state')
+            self._logger.debug(f'Space created, with state={state}.')
+            space_id = client.spaces.get_uid(space_details)
         self._logger.info(f'Space id: {space_id}')
         return space_id
 
